package com.rjconsultores.ventaboletos.entidad;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import com.rjconsultores.ventaboletos.vo.parada.ConexionCtrlVO;

/**
 * 
 * @author wallace
 */
@Entity
@SequenceGenerator(name = "CONEXION_RUTA_TRAMO_CTRL_SEQ", sequenceName = "CONEXION_RUTA_TRAMO_CTRL_SEQ", allocationSize = 1)
@Table(name = "CONEXION_RUTA_TRAMO_CTRL")
public class ConexionRutaTramoCtrl implements Serializable {
	private static final long serialVersionUID = 1L;
	@Id
	@Basic(optional = false)
	@GeneratedValue(strategy = GenerationType.AUTO, generator = "CONEXION_RUTA_TRAMO_CTRL_SEQ")
	@Column(name = "CONEXIONRUTATRAMOCTRL_ID")
	private Long conexionRutaTramoId;
	@Column(name = "ACTIVO")
	private Boolean activo;
	@Column(name = "FECMODIF")
	@Temporal(TemporalType.TIMESTAMP)
	private Date fecmodif;
	@Column(name = "USUARIO_ID")
	private Integer usuarioId;
	
	@ManyToOne
	@JoinColumn(name = "ORIGEN_ID")
	private Parada origen;
	
	@ManyToOne
	@JoinColumn(name = "DESTINO_ID")
	private Parada destino;

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "CONEXIONRUTACTRL_ID")
	private ConexionRutaCtrl conexionRutaCtrl;
	
	@ManyToOne
	@JoinColumn(name = "RUTA_ID")
	private Ruta ruta;

	@Transient
	private Short secuencia;
	@Transient
	private boolean valido;
	@Transient
	private Integer grupo;
	
	@Transient
	private ConexionCtrlVO conexionCtrl;
	
	public ConexionRutaTramoCtrl() {
		super();
	}
	
	public ConexionRutaTramoCtrl(Long conexionRutaTramoId) {
		this();
		this.conexionRutaTramoId = conexionRutaTramoId;
	}

	public Boolean getActivo() {
		return activo;
	}

	public Long getConexionRutaTramoId() {
		return conexionRutaTramoId;
	}

	public void setConexionRutaTramoId(Long conexionRutaTramoId) {
		this.conexionRutaTramoId = conexionRutaTramoId;
	}

	public void setActivo(Boolean activo) {
		this.activo = activo;
	}

	public Date getFecmodif() {
		return fecmodif;
	}

	public void setFecmodif(Date fecmodif) {
		this.fecmodif = fecmodif;
	}

	public Integer getUsuarioId() {
		return usuarioId;
	}

	public void setUsuarioId(Integer usuarioId) {
		this.usuarioId = usuarioId;
	}

	public ConexionRutaCtrl getConexionRutaCtrl() {
		return conexionRutaCtrl;
	}

	public void setConexionRutaCtrl(ConexionRutaCtrl conexionRutaCtrl) {
		this.conexionRutaCtrl = conexionRutaCtrl;
	}

	public Short getSecuencia() {
		return secuencia;
	}

	public void setSecuencia(Short secuencia) {
		this.secuencia = secuencia;
	}

	public boolean isValido() {
		return valido;
	}

	public void setValido(boolean valido) {
		this.valido = valido;
	}

	public Integer getGrupo() {
		return grupo;
	}

	public void setGrupo(Integer grupo) {
		this.grupo = grupo;
	}

	public static Long getConexionRutaTramoConexionId(List<Conexion> conexiones) {
		if(conexiones != null && !conexiones.isEmpty()) {
			Conexion conexionRutaTramoCtrl = conexiones.get(0);
			return conexionRutaTramoCtrl.getConexionRutaTramoId();
		}
		return null;

	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((conexionRutaTramoId == null) ? 0 : conexionRutaTramoId.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof ConexionRutaTramoCtrl))
			return false;
		ConexionRutaTramoCtrl other = (ConexionRutaTramoCtrl) obj;
		if (getConexionRutaTramoId() == null) {
			if (other.getConexionRutaTramoId() != null)
				return false;
		} else if (!getConexionRutaTramoId().equals(other.getConexionRutaTramoId()))
			return false;
		return true;
	}

	public Parada getOrigen() {
		return origen;
	}

	public void setOrigen(Parada origen) {
		this.origen = origen;
	}

	public Parada getDestino() {
		return destino;
	}

	public void setDestino(Parada destino) {
		this.destino = destino;
	}

	public Ruta getRuta() {
		return ruta;
	}

	public void setRuta(Ruta ruta) {
		this.ruta = ruta;
	}

	public ConexionCtrlVO getConexionCtrl() {
		return conexionCtrl;
	}

	public void setConexionCtrl(ConexionCtrlVO conexionCtrl) {
		this.conexionCtrl = conexionCtrl;
	}
}
