package com.rjconsultores.ventaboletos.service.impl;

import java.util.Calendar;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.itextpdf.text.pdf.PdfStructTreeController.returnType;
import com.rjconsultores.ventaboletos.dao.CategoriaFormAutorizacaoDAO;
import com.rjconsultores.ventaboletos.entidad.CategoriaFormAutorizacao;
import com.rjconsultores.ventaboletos.service.CategoriaFormAutorizacaoService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

@Service("CategoriaFormAutorizacaoService")
public class CategoriaFormAutorizacaoServiceImpl  implements CategoriaFormAutorizacaoService{
	
	@Autowired
    private CategoriaFormAutorizacaoDAO categoriaFormAutorizacaoDAO;
	private static Logger log = LogManager.getLogger(CategoriaFormAutorizacaoServiceImpl.class);


	@Override
	public List<CategoriaFormAutorizacao> obtenerTodos() {
		return categoriaFormAutorizacaoDAO.obtenerTodos();
	}

	@Override
	public CategoriaFormAutorizacao obtenerID(Integer id) {
		return categoriaFormAutorizacaoDAO.obtenerID(id);
	}

	@Transactional
	public CategoriaFormAutorizacao suscribir(CategoriaFormAutorizacao entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
        entidad.setFecmodif(Calendar.getInstance().getTime());
        entidad.setActivo(Boolean.TRUE);
    	
		return categoriaFormAutorizacaoDAO.suscribir(entidad);
	}

	@Transactional
	public CategoriaFormAutorizacao actualizacion(CategoriaFormAutorizacao entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
        entidad.setFecmodif(Calendar.getInstance().getTime());
        entidad.setActivo(Boolean.TRUE);
		return categoriaFormAutorizacaoDAO.actualizacion(entidad);
	}

	@Transactional
    public CategoriaFormAutorizacao salvar(CategoriaFormAutorizacao entidad) {
    	entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
        entidad.setFecmodif(Calendar.getInstance().getTime());
        entidad.setActivo(Boolean.TRUE);

        return categoriaFormAutorizacaoDAO.saverOrUpdate(entidad);
    }
	
	@Transactional
	public void borrar(CategoriaFormAutorizacao entidad) {
		categoriaFormAutorizacaoDAO.borrar(entidad);
	}

	@Override
	public List<CategoriaFormAutorizacao> obtenerTodosPorCategoria(Integer categoriaId) {
		return categoriaFormAutorizacaoDAO.obtenerTodosPorCategoria(categoriaId);
	}

	@Override
	public CategoriaFormAutorizacao pesquisarPorCategoriaEmpresa(Integer categoriaId, Integer empresaId) {
		return categoriaFormAutorizacaoDAO.pesquisarPorCategoriaEmpresa(categoriaId, empresaId);
	}
}
