package com.rjconsultores.ventaboletos.service.impl;

import java.util.Calendar;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.rjconsultores.ventaboletos.dao.ExcecaoGrupoContratoDAO;
import com.rjconsultores.ventaboletos.dao.GrupoContratoDAO;
import com.rjconsultores.ventaboletos.dao.TarifaGrupoContratoDAO;
import com.rjconsultores.ventaboletos.entidad.ExcecaoGrupoContrato;
import com.rjconsultores.ventaboletos.entidad.GrupoContrato;
import com.rjconsultores.ventaboletos.entidad.TarifaGrupoContrato;
import com.rjconsultores.ventaboletos.service.GrupoContratoService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;


@Service("grupoContratoService")
public class GrupoContratoServiceImpl implements GrupoContratoService {

    @Autowired
    private GrupoContratoDAO grupoContratoDAO;
    
    @Autowired
    private TarifaGrupoContratoDAO tarifaGrupoContratoDAO;
    
    @Autowired
    private ExcecaoGrupoContratoDAO excecaoGrupoContratoDAO;

    public List<GrupoContrato> obtenerTodos() {
        return grupoContratoDAO.obtenerTodos();
    }

    public GrupoContrato obtenerID(Long id) {
        return grupoContratoDAO.obtenerID(id);
    }

    @Transactional
    public GrupoContrato suscribir(GrupoContrato entidad) {
        entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
        entidad.setFecmodif(Calendar.getInstance().getTime());
        entidad.setActivo(Boolean.TRUE);

        return grupoContratoDAO.suscribir(entidad);
    }

    @Transactional
    public GrupoContrato actualizacion(GrupoContrato entidad) {
        entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
        entidad.setFecmodif(Calendar.getInstance().getTime());
        entidad.setActivo(Boolean.TRUE);

        return grupoContratoDAO.actualizacion(entidad);
    }

    @Transactional
    public void borrar(GrupoContrato entidad) {
        entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
        entidad.setFecmodif(Calendar.getInstance().getTime());
        entidad.setActivo(Boolean.FALSE);

        grupoContratoDAO.actualizacion(entidad);
    }

    public List<GrupoContrato> buscar(String descgrupo) {
        return grupoContratoDAO.buscar(descgrupo);
    }

    @Transactional
	public boolean gravarTarifasGrupoContrato(List<TarifaGrupoContrato> tarifas) {
		return tarifaGrupoContratoDAO.gravarTarifasGrupoContrato(tarifas);
	}
    
    @Transactional
	public boolean gravarExcecoesGrupoContrato(List<ExcecaoGrupoContrato> excecoes) {
		return excecaoGrupoContratoDAO.gravarExcecoesGrupoContrato(excecoes);
	}
}
